import os
import yaml
import ROOT
import numpy as np
from copy import deepcopy

ROOT.gROOT.ProcessLine("gErrorIgnoreLevel = 1001;") # Kill some CHI2 warnings
ROOT.RooMsgService.instance().setGlobalKillBelow(ROOT.RooFit.WARNING) # Kill some long RooFit Info log

# Load the morphing #

morph_file = ROOT.TFile("morphing.root")

workspace = deepcopy(morph_file.Get("MorphWorkspace"))
delayVar  = workspace.var("delayVar")
bxidVar   = workspace.var("bxidVar")
morphing  = workspace.pdf("morph")  
morphing.useHorizontalMorphing(False) # False works fine, True produces memory leak

morph_file.Close()

# Function to estimate the reco delay #

def getRecoDelay(recoHist):
    bxRange = np.arange(0.,50.,0.1)
    chi2 = []
    for i in range(bxRange.shape[0]):
        delayVar.setVal(bxRange[i])
        trueHist = morphing.createHistogram("trueHist{:.2f}".format(bxRange[i]),bxidVar) 
        chi2.append(recoHist.Chi2Test(trueHist,"UU CHI2"))
        trueHist.Delete()
    chi2 = np.array(chi2)
    return bxRange[chi2.argmin()]

# Load the histograms #

hist_file = ROOT.TFile("hists.root")

TH1_dir = "DQMData/Run 1/Ph2TkBXHist/Run summary/Hist1D"

with open("hists.yml",'r') as handle:
    histnames = yaml.load(handle)

# Run the "algo" #
for name,true_delay in histnames.items():
    h = hist_file.Get(TH1_dir+'/'+name)
    reco_delay = getRecoDelay(h)
    print (f'True = {true_delay:0.3f}, reco = {reco_delay:0.3f}')

hist_file.Close()
